#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::util::users - Cached getpwent data
#
package Security::util::users;

use User::pwent;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check() &get() );

our ($uids, $users);

sub init {
	setpwent;
	while (my $pw = getpwent) {
		# Skip the YP map entry...
		if ( defined $pw && $pw->name ne '+' ) {
			$users->{$pw->name} = $pw;
			$uids->{$pw->uid} = $pw;
		}
	}
	endpwent;
}

sub _byuid {
	$users->{$a}->uid <=> $users->{$b}->uid;
}

sub get {
	my ($field, $value, $subfield) = @_;

	if ( defined $value && defined $subfield ) {
		return $field eq 'uid' 
			? $uids->{$value}->$subfield
			: $users->{$value}->$subfield;
	} else {
		return $field eq 'uid' 
			? sort keys %{$uids}
			: sort _byuid keys %{$users};
	}
}

sub check {
	my ($field, $value) = @_;
	my ($res);

	if ( $field eq 'uid' ) {
		$res = exists $uids->{$value};
	} else {
		$res = exists $users->{$value};
	}

	return $res;
}

1;

__END__

=head1 NAME

Security::util::users - module for checking against the password file.

=head1 SYNOPSIS

Security::util::users::check($field, $value)

=head1 DESCRIPTION

A wrapper routine around C<User::pwent::getpwent> calls to return the
password file information.

=head1 API

The API calls provided by this module

=over 4

=item C<init ()>

Reads in all the password information.

=item C<get ($field [, $value, $subfield])>

If I<$field> is the only argument, C<get()> will return a list all the
values, in uid order.  I<$field> can be either C<name> or C<uid>.  If
I<$value> and I<$subfield> are defined, C<get()> will treat $value as
either a login name or uid and return the specific $subfield value from
the password file.  

=item C<check ($field, $value)>

Returns true if I<$field> has I<$value>.  I<$field> can be either C<name>
or C<uid>.

=back

=head1 CONFIGURATION OPTIONS

None.

=head1 SEE ALSO

L<User::pwent>

L<perl>

=cut
